package jain.application.services.jcc;

/*
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  Copyrights:
 *
 *  Copyright - 2001 Sun Microsystems, Inc. All rights reserved.
 *  901 San Antonio Road, Palo Alto, California 94043, U.S.A.
 *
 *  This product and related documentation are protected by copyright and
 *  distributed under licenses restricting its use, copying, distribution, and
 *  decompilation. No part of this product or related documentation may be
 *  reproduced in any form by any means without prior written authorization of
 *  Sun and its licensors, if any.
 *
 *  RESTRICTED RIGHTS LEGEND: Use, duplication, or disclosure by the United
 *  States Government is subject to the restrictions set forth in DFARS
 *  252.227-7013 (c)(1)(ii) and FAR 52.227-19.
 *
 *  The product described in this manual may be protected by one or more U.S.
 *  patents, foreign patents, or pending applications.
 *
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  Author:
 *
 *  Telcordia Technologies, Inc.
 *  Farooq Anjum, John-Luc Bakker, Ravi Jain
 *  445 South Street
 *  Morristown, NJ 07960
 *
 *  Version       : 1.0
 *  Notes         :
 *
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

import jain.application.services.jcp.*;

/**
    The JccCall interface extends the JcpCall interface of JCP. 
    This interface provides additional methods on a Call. Further, the state machine 
    on the JccCall is also similar to the state machine of the JcpCall except for an
    extra transition as shown in the following figure. 
     <CENTER>
     <H1>JccCall FSM</H1>
     <IMG SRC="doc-files/callstates.gif" ALIGN="center">
     </CENTER>
     
       <H5>Event Snapshots</H5> 
       By default, when an listener is added 
       to a telephone call, the first batch of events may be a "snapshot".
       That is, if the listener was added after state changes in the Call,
       the first batch of events will inform the application of the 
       current state of the Call. Note that these snapshot events do NOT 
       provide a history of all events on the Call, rather they provide 
       the minimum necessary information to bring the application 
       up-to-date with the current state of the Call. 
     

*/
public interface JccCall extends JcpCall
{

    /**
       Add a listener to this call. This also reports all state changes in the state of 
       the JccCall and {@link JccConnection} objects. The listener added with this method will 
       report events on the call for as long as the implementation can listen to 
       the JccCall. In the case that 
       <OL>
       <LI>the implementation can no longer observe the JccCall
       <LI>this listener has been removed from this JccCall
       <LI>the JccCall has completed and moved to the {@link JcpCall#INVALID} 
       state the application receives a 
       {@link JcpCallEvent#CALL_EVENT_TRANSMISSION_ENDED} event. 
       </OL>

        <H5> Connection Events </H5>
        A JccConnectionEvent extends JccCallEvent and a JccConnectionListener extends
        JccCallListener. Hence, a JccConnectionListener can also be added using this
        method. In such a case, events on the JccConnection objects associated with this
        JccCall object might have to be reported to the registered listener after
        consulting with the EventFilter. 
        In summary  
        the implementation must distinguish between JccCallListener, 
         and JccConnectionListener in order to  determine the events for which 
         the implementation must consult the corresponding EventFilter. 

        <H5>Multiple Invocations</H5> 
       Registering a single listener twice will result in the
       application being notified of the  events specified in the filter 
       implementation used in the last 
       registration. Note that, because of this,  
       registering for the same event multiple times should not
       result in multiple notifications being sent to an application for
       the same event.

       <P><B>Post-Conditions:</B>
       <OL>
        <LI>A snapshot of events is delivered to the listener, if appropriate. 
       </OL>

       @param calllistener JccCallListener object that receives the specified events.
       @param filter EventFilter which determines if the event is to be 
       delivered to the specified listener.
        @throws MethodNotSupportedException The listener cannot be added at this time.
        @throws ResourceUnavailableException The resource limit for the number of 
        listeners has been exceeded. 
     */
    public void addCallListener( JccCallListener calllistener, EventFilter filter)     throws ResourceUnavailableException, 
    MethodNotSupportedException;


    /**
       Add a connection listener to all connections under this call. 
       This method applies to any JcpConnection 
       that already exists when the method is called as well as any JcpConnections 
       that will be added in the future to this JccCall object.
       This method
       behaves similar to the addCallListener(conListener, filter) method 
       on this interface where conListener is in fact a JccConnectionListener. 
       When conListener is not a JccConnectionListener but is only a JccCallListener, 
       a similar behavior as addCallListener(conListener, newfilter)
       can be obtained with this method using a JccConnectionListener and a 
       different EventFilter which filters
       all the JccConnection related events in addition to the events being filtered
       by newfilter. 
       Note though that using this method only 
       JccConnectionListeners can be added. 
       <p>

       Registering a single listener twice will result in the
       last filter being used for the purposes of consultation to 
       determine the events that the application is interested in.
       Note that registering for the same event multiple times should not
       result in multiple notifications being sent to an application for
       the same event.

       <P><B>Post-Conditions:</B>
       <OL>
        <LI>A snapshot of events is delivered to the listener, if appropriate. 
       </OL>

       @param cl JccConnectionListener object that receives the specified events.
       @param filter EventFilter determines if the event is to be 
       delivered to the specified listener. 
        @throws MethodNotSupportedException The listener cannot be added at this time.
        @throws ResourceUnavailableException The resource limit for the number of 
        listeners has been exceeded. 
       <br>
       */
     
    public void addConnectionListener( JccConnectionListener cl, EventFilter filter )     throws ResourceUnavailableException, 
    MethodNotSupportedException; 

    /**
    Removes the connection listener from all connections under this call. Note that 
    if the listener is currently not registered then this method fails silently. 
    
     <P><B>Post-Conditions:</B> <OL> 
     <LI>CALL_EVENT_TRANSMISSION_ENDED is delivered to the 
     application </OL>
    
    @param cl JccConnectionListener object that was registered using a corresponding
    addConnectionListener method. 
    */

    public void removeConnectionListener( JccConnectionListener cl);

    /**
        This method requests the release of the call object and associated connection
        objects. Thus this method is equivalent to using the {@link JccConnection#release()}
        method on each JccConnection which is part of the Call. Typically each JccConnection 
        associated with this call will move into the {@link JccConnection#DISCONNECTED} state. 
        The call will also be terminated in the network. If the application
        has registered as a listener then it receives the {@link JcpCallEvent#CALL_EVENT_TRANSMISSION_ENDED}
        event.  

       <p> <B>Pre-conditions:</B> <OL>
        <LI>(this.getProvider()).getState() == IN_SERVICE <br>
        <LI> this.getState() == ACTIVE
        </OL>
       <p> <B>Post-conditions:</B> <OL>
        <LI>(this.getProvider()).getState() == IN_SERVICE <br>
        <LI> this.getState() == INVALID
        <LI>CALL_EVENT_TRANSMISSION_ENDED event delivered to the 
        valid Calllisteners. 
        <LI>Appropriate ConnectionEvents are also delivered to the ConnectionListeners. 
        </OL>

        @throws PrivilegeViolationException The application does not have 
        the authority or permission to disconnect the Call. For example, 
         an  Address associated with this Call may not be controllable 
        in the Provider's domain. 
        @throws ResourceUnavailableException An internal resource required 
        to drop a connection is not available. 
        @throws InvalidStateException Some object required for the 
        successful invocation of this method is not in the proper state as 
        given by this method's pre-conditions. 
    */
    public void release() throws PrivilegeViolationException, 
    ResourceUnavailableException, InvalidStateException;



    /**
        Creates (a) new JccConnection(s) and attaches it/them to this JccCall. The JccConnection
        objects is/are associated with (an) JccAddress object(s) corresponding to the 
        string(s) given as (an) input parameter(s). Note that following 
        this operation the returned JccConnection object might have to be routed to the 
        JccAddress which can be accomplished using the {@link JccConnection#routeConnection(boolean)}. 

		<h5><A NAME="createConnectionSemantics">Semantics</a></h5>
		<EM>We recognize that these semantics are awkward. As the alignment process with other Call Control APIs continues, we might change these semantis in future versions of this API.</EM><p>
		Consider party A calling party B in case of a first party call. In case of a third party call, 
		consider party A and party B being connected. Below the behavior of this method is exemplified:
		<UL>
		<LI><code><b>if</b> (this.getState()==IDLE) <b>then</b> createConnection("B","A",null,null) <b>{</b></code><blockquote>
		
		 will result in a first party call-setup. 
		Both the terminating and originating connections will be created. However, a reference to the  originating 
		connection i.e "A"'s connection will be returned. Hence, a getAddress() method invoked 
		on the returned reference results in the JcpAddress corresponding to party 
		"A" being returned. Note though that both party "A" and party "B" would have to be routed to 
		using the JccConnection.routeConnection() method on the JccConnection
		object corresponding to "A" and "B", respectivelly. The JccConnection reference of "B" can be obtained by invoking the 
		getConnections() method on this JccCall object.
		
		</blockquote><code><b>}</b> <b>else if</b> (this.getState()==ACTIVE) <b>then</b> createConnection("B","A",null,null) <b>{</b></code><blockquote>
		
		will result in  third party setup of a Connection.
		A reference to the  created connection i.e "B"'s connection will be returned.
		A getAddress() method invocation on the returned JccConnection reference object
		results in the JcpAddress corresponding to party "B" being returned.
		Party "B" will have to be routed to in this case.	    
		
		</blockquote><code><b>}</b></code>
		<LI><code>createConnection("A","A",null,null)</code> will result in third party setup of the "originating" Connection.
	    A getAddress() method invocation on the returned JccConnection reference object
	    results in the JcpAddress corresponding to party "A" being returned. 
	    Party "A" will have to be routed to in this case.	    
		<LI><code>createConnection("C","A","B","B")</code> will be applied in case the number of party B is translated into
		the number of party C, while party A dialed party B. <code>createConnection("C","A","B",null)</code> results in the 
		same behavior.
		A reference to the  created connection i.e "C"'s connection will be returned.
		A getAddress() method invocation on the returned JccConnection reference object
	    results in the JcpAddress corresponding to party "C" being returned.
		Party "C" will have to be routed to in this case.	    
		<LI><code>createConnection("C","A","B","D")</code> will be applied in case the number of party D is translated into
		the number of party C, while party A dialed party B.
		A reference to the  created connection i.e "C"'s connection will be returned.
		A getAddress() method invocation on the returned JccConnection reference object
	    results in the JcpAddress corresponding to party "C" being returned.
		Party "C" will have to be routed to in this case.	    
	    </UL>

       <p> <B>Pre-conditions:</B> <OL>
        <LI>(this.getProvider()).getState() == IN_SERVICE <br>
        <LI> this.getState() != INVALID
        </OL>
       <p> <B>Post-conditions:</B> <OL>
       <LI> let conn = createConnection(..); <br>
        <LI> conn.getState() == IDLE state<br>
        <LI> this.getState() == ACTIVE
        <LI>(this.getProvider()).getState() == IN_SERVICE <br>
        </OL>

        @return one of the JccConnection objects created.

        @param targetAddress specifies the address to which the call 
        should be routed.
        @param originatingAddress specifies the address of the originating (calling)
        party for this leg of the call.
        @param originalCalledAddress specifies the initial destination address to which
        this leg of the call was initiated. This is optional and can be set to null.
        @param redirectingAddress specifies the last address from which this leg of the call
        was redirected. This is optional and can be set to null. 

		@throws InvalidArgumentException is case an either or both the targetAddress or the originatingAddress is set to null.
        @throws InvalidStateException Some object required by this method is not 
        in a valid state as designated by the pre-conditions for this method.
        @throws ResourceUnavailableException An internal resource necessary
        for creating the Connection object is unavailable. 
        @throws PrivilegeViolationException The application does not have 
        the proper authority to create the Connection.
        @throws MethodNotSupportedException The implementation does not 
        support this method
        @throws InvalidPartyException In case one of the parties given in invalid.

    */
    public JccConnection createConnection(String targetAddress, String originatingAddress, 
    String originalCalledAddress, String redirectingAddress)         throws InvalidStateException, 
    ResourceUnavailableException, PrivilegeViolationException, 
    MethodNotSupportedException, InvalidArgumentException, InvalidPartyException;  
    
    /**
    This method requests routing of a call to the targetAddress given as an input parameter.
    
    This results 
    in the creation of (a) JccConnection object(s) associated with this JccCall. 
    The JccConnection
    object(s) is/are associated with an address object(s) passed as a parameter(s). Note that the 
    addresses are passed as strings. The implementation is expected to find the JccAddress 
    object corresponding to the string assuming that the JccAddress is local to the JccProvider.
    The given string may not correspond to any JccAddress object in the JccProvider's 
    domain which would be the case for a call to a remote Address.     
    This method
    is equivalent to the {@link #createConnection(String,String,String,String)}, 
    <a href="JccConnection.html#routeConnection(boolean)">JccConnection.routeConnection(FALSE)</A>
    and {@link JccConnection#attachMedia()} or is also
    equivalent to {@link #createConnection(String,String,String,String)} and 
    <a href="JccConnection.html#routeConnection(boolean)">JccConnection.routeConnection(TRUE)</A>. 

	<h5><A NAME="routeCallSemantics">Semantics</a></h5>
	<EM>We recognize that these semantics are awkward. As the alignment process with other Call Control APIs continues, we might change these semantis in future versions of this API.</EM><p>
	Consider party A calling party B in case of a first party call. In case of a third party call, 
	consider party A and party B being connected. Below the behavior of this method is exemplified:
		<UL>
				
		<LI><code><b>if</b> (this.getState()==IDLE) <b>then</b> routeCall("B","A",null,null) <b>{</b></code><blockquote>

		will result in a first party call-setup. 
		Both the terminating and originating connections will be created. However, a reference to the  originating 
		connection i.e "A"'s connection will be returned. Hence, a getAddress() method invoked 
		on the returned reference results in the JcpAddress corresponding to party 
		"A" being returned.

		</blockquote><code><b>}</b> <b>else if</b> (this.getState()==ACTIVE) <b>then</b> routeCall("B","A",null,null) <b>{</b></code><blockquote>

		will result in  third party setup of a Connection.
        A reference to the  created connection i.e "B"'s connection will be returned.
		A getAddress() method invocation on the returned JccConnection reference object
	    results in the JcpAddress corresponding to party "B" being returned.
	    Party "B" will have to be routed to in this case. 

		</blockquote><code><b>}</b></code>
		<LI><code>routeCall("A","A",null,null)</code> will result in third party setup of the "originating" Connection.
	    A getAddress() method invocation on the returned JccConnection reference object
	    results in the JcpAddress corresponding to party "A" being returned. Note
	    also that party "A" will have to be routed to in this case. 
		<LI><code>routeCall("C","A","B","B")</code> will be applied in case the number of party B is translated into
		the number of party C, while party A dialed party B. <code>routeCall("C","A","B",null)</code> results in the 
		same behavior.
		A reference to the  created connection i.e "C"'s connection will be returned.
		A getAddress() method invocation on the returned JccConnection reference object
	    results in the JcpAddress corresponding to party "C" being returned.
		<LI><code>routeCall("C","A","B","D")</code> will be applied in case the number of party D is translated into
		the number of party C, while party A dialed party B.
		A reference to the  created connection i.e "C"'s connection will be returned.
		A getAddress() method invocation on the returned JccConnection reference object
	    results in the JcpAddress corresponding to party "C" being returned.
        </UL>
       <p> <B>Pre-conditions:</B> <OL>
        <LI>(this.getProvider()).getState() == IN_SERVICE <br>
        <LI> this.getState() != INVALID
        </OL>
       <p> <B>Post-conditions:</B> <OL>
        <LI>conn.getState() != IDLE or AUTHORIZE_CALL_ATTEMPT or DISCONNECTED
        where conn is the object returned as a result of this method.<br>
        <LI>this.getState() == ACTIVE<br>
        <LI>(this.getProvider()).getState() == IN_SERVICE <br>
        </OL>
        
        @param targetAddress specifies the address to which the call 
        should be routed.
        @param originatingAddress specifies the address of the originating (calling)
        party for this leg of the call.
        @param originalCalledAddress specifies the initial destination address to which
        this leg of the call was initiated. This is optional and can be set to null.
        @param redirectingAddress specifies the last address from which this leg of the call
        was redirected. This is optional and can be set to null. 

		@throws InvalidArgumentException is case an either or both the targetAddress or the originatingAddress is set to null.
        @throws InvalidStateException Some object required by this method is not 
        in a valid state as designated by the pre-conditions for this method.
        @throws ResourceUnavailableException An internal resource necessary
        for creating the Connection object is unavailable. 
        @throws PrivilegeViolationException The application does not have 
        the proper authority to create the Connection.
        @throws MethodNotSupportedException The implementation does not 
        support this method
        @throws InvalidPartyException In case one of the parties given in invalid. 
        @return one of the JccConnection objects created
    
    */
    public JccConnection routeCall(String targetAddress, String originatingAddress, 
    String originalDestinationAddress, String redirectingAddress) throws InvalidStateException, 
    ResourceUnavailableException, PrivilegeViolationException,MethodNotSupportedException, 
    InvalidPartyException, InvalidArgumentException;  


    /**
    The application calls this method to supervise a call. The application can set a 
    granted connection time for this call. If an application calls this function before
    it calls a {@link #routeCall(String,String,String,String)}, the timer measurement will start as soon as 
    the call is answered by the called party. 
    <p>
    The call supervision events are then reported to the listener registered using this method.
    The events related to call supervision are reported only to the listeners registered using this
    method. 
    <p>
    <em> Note </em> that a policy object may be designed to define the policy to be 
    implemented by the platform as a result of this method instead of defining 
    the policy through the given parameters. This might be designed in the 
    future specifications. 

    
    @param calllistener JccCallListener object that receives the specified events.
    @param time specifies the granted time in milliseconds for the connection. When specified
    as 0, volume based supervision is applied. Either bytes(volume) or time should be specified.
    @param treatment defines the treatment of the call by the call control service when 
    the call supervision timer expires. The values which may be combined using a logical
    OR function are:
    <OL>
    <LI><code>0x01</code> to release the call when the call supervision timer expires.<br> 
    <LI><code>0x02</code> to notify the application when the call supervision timer expires. <br>
    <LI><code>0x04</code> to send a warning tone to the controlling party when a call supervision timer
    expires. If call release is requested, then the call will be released following the 
    tone after an administered time period. 
    </OL>
    @param bytes specifies the granted number of bytes that can be transmitted for the 
    connection. When the quantity is specified as 0, time based supervision is applied. 
    
    @throws MethodNotSupportedException if the implementation does not support this
    method. 
    */
    public void superviseCall(JccCallListener calllistener,double time,int treatment, double bytes) throws 
    MethodNotSupportedException;
    
}
 
