package jain.application.services.jcp;

/*
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  Copyrights:
 *
 *  Copyright - 2001 Sun Microsystems, Inc. All rights reserved.
 *  901 San Antonio Road, Palo Alto, California 94043, U.S.A.
 *
 *  This product and related documentation are protected by copyright and
 *  distributed under licenses restricting its use, copying, distribution, and
 *  decompilation. No part of this product or related documentation may be
 *  reproduced in any form by any means without prior written authorization of
 *  Sun and its licensors, if any.
 *
 *  RESTRICTED RIGHTS LEGEND: Use, duplication, or disclosure by the United
 *  States Government is subject to the restrictions set forth in DFARS
 *  252.227-7013 (c)(1)(ii) and FAR 52.227-19.
 *
 *  The product described in this manual may be protected by one or more U.S.
 *  patents, foreign patents, or pending applications.
 *
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  Author:
 *
 *  Telcordia Technologies, Inc.
 *  Farooq Anjum, John-Luc Bakker, Ravi Jain
 *  445 South Street
 *  Morristown, NJ 07960
 *
 *  Version       : 1.0
 *  Notes         :
 *
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**
 * A <CODE>JcpProvider</CODE> represents the telephony software-entity that
 * interfaces with a 
 * telephony subsystem.
 
 * <H4>Introduction</H4>
 *
 * The telephony subsystem could be a PBX connected to 
 * a server machine, a telephony/fax card in a desktop machine or a networking
 * technology such as IP or ATM.
 *
 * <H4>JcpProvider States</H4>
 * 
 * The JcpProvider may either be in one of the following states:
 * {@link #IN_SERVICE}, {@link #OUT_OF_SERVICE}, or
 * {@link #SHUTDOWN}. The JcpProvider state represents whether any
 * action on that JcpProvider may be valid. The following tables describes each
 * state:
 * <p>
 * <TABLE>
 * <TR>
 * <TD WIDTH="20%">
 * {@link #IN_SERVICE}
 * </TD>
 * <TD WIDTH="80%">
 * This state indicates that the JcpProvider is currently alive and available for
 * use.
 * </TD>
 * </TR>
 *
 * <TR>
 * <TD WIDTH="20%">
 * {@link #OUT_OF_SERVICE}
 * </TD>
 * <TD WIDTH="80%">
 * This state indicates that a JcpProvider is temporarily not available for use.
 * Many methods in this API are invalid when the JcpProvider is in
 * this state. JcpProviders may come back in service at any time, however, the
 * application can take no direct action to cause this change.
 * </TD>
 * </TR>
 *
 * <TR>
 * <TD WIDTH="20%">
 * {@link #SHUTDOWN}:
 * </TD>
 * <TD WIDTH="80%">
 * This state indicates that a JcpProvider is permanently no longer available for
 * use. Most methods in the  API are invalid when the JcpProvider
 * is in this state. Applications may use the {@link #shutdown()}
 * method on this interface to cause a JcpProvider to move into the
 * {@link #SHUTDOWN} state.
 * </TD>
 * </TR>
 * </TABLE>
 * <p>
 * The following diagram shows the allowable state transitions for the
 * JcpProvider.
 * <CENTER>
 *  <H1>Provider FSM</H1>
 * <IMG SRC="doc-files/providerstates.gif" ALIGN="center">
 * </CENTER>
 * <H4>Obtaining a JcpProvider</H4>
 *
 * A JcpProvider is created and returned by the
 * {@link JcpPeer#getProvider(String)} method which is given a string to
 * describe the desired JcpProvider. This method sets up any needed communication
 * paths between the application and the JcpProvider. The string given is one of
 * the services listed in the {@link JcpPeer#getServices()}.
 * 
 * <H4>Listeners and Events</H4>
 *
 * Each time a state changes occurs on a JcpProvider, the application is notified
 * via an <EM>event</EM>. This event is reported via the
 * {@link JcpProviderListener} interface. Applications instantiate objects
 * which implement this interface and use the
 * {@link #addProviderListener(JcpProviderListener)} method to begin the delivery of
 * events.
 * Applications may then query the event object
 * returned for the specific state change, via the {@link JcpEvent#getID()}
 * method.
 * When the JcpProvider changes state,
 * a {@link JcpProviderEvent}
 * is sent to the JcpProviderListener, having one of the following event ids:
 * {@link JcpProviderEvent#PROVIDER_IN_SERVICE}, {@link JcpProviderEvent#PROVIDER_OUT_OF_SERVICE},
 * and {@link JcpProviderEvent#PROVIDER_SHUTDOWN}.
 * A <CODE>JcpProviderEvent</CODE> with event id 
 * The {@link JcpProviderEvent#PROVIDER_EVENT_TRANSMISSION_ENDED} is delivered to all
 * JcpProviderListeners
 * when the JcpProvider
 * becomes unobservable and is the final event delivered to the listener. When a 
 JcpProvider becomes unobservable, then implementation would not be sure of the 
 state of the JcpProvider.
 *
 * <H4>Call Objects and Providers</H4>
 *
 * Applications may create a {@link JcpCall} object representing new calls using the
 * {@link #createCall()} method. A new JcpCall is returned in the
 * {@link JcpCall#IDLE} state. Applications may then use this idle JcpCall to
 * place new telephone calls. 
 *
 * <H4>Address Objects</H4> 
 A  {@link JcpAddress} object represents what we
 * commonly think of as a "telephone number."  
 Unlike JcpCall objects, applications may not create JcpAddress 
 objects. An address is obtained through {@link #getAddress(String)}
 * <H4>Multiple Providers and Multiple Applications</H4>
 *
 * It is not guaranteed or expected that objects
 * instantiated through one JcpProvider will be usable with another JcpProvider.
 * Therefore, an application that uses two providers must keep all the objects
 * relating to these providers separate. In the future, there may be a
 * mechanism whereby a JcpProvider may share objects with another JcpProvider if
 * they
 * are speaking to the same telephony hardware, however, such capabilities are
 * not available in this release.
 * <p>
 * Also, multiple applications may request and communicate with the same
 * JcpProvider implementation. Typically, since each application executes in its
 * own object space, each will have its own instance of the JcpProvider object.
 * These two different JcpProvider objects may, in fact, be proxies for a
 * centralized JcpProvider instance. Methods in JCP like {@link #shutdown()} 
 are specified to
 * affect only the invoking applications and have no affect on others. 

<h5>Event Snapshots</h5>

By default, when a {@link JcpProviderListener} is added through {@link #addProviderListener(JcpProviderListener)},
the first batch of events may be a "snapshot".
That is, if the listener was added after state changes in the Provider, the first batch of events will inform the
application of the current state of the Provider. Note that these snapshot events do NOT provide a history
of all events on the Provider, rather they provide the minimum necessary information to bring the application
up-to-date with the current state of the Provider. 

 * @see JcpPeer
 * @see JcpPeerFactory
 * @see JcpProviderListener
 */
public interface JcpProvider
{
    /**
       Returns the state of the JcpProvider.
       <br>
       <br>
       @return Integer representing the state of the provider. See static int's defined in this object.
     */
    public int getState();

    /**
       Creates a new instance of the call with no connections. The new call object is 
       in the {@link JcpCall#IDLE} state. An exception is generated if a new call cannot be
       created for various reasons. This JcpProvider must be in the {@link JcpProvider#IN_SERVICE} 
       state, otherwise an {@link InvalidStateException} is thrown. 
       
	   <p><B>Pre-conditions:</B><OL>
        <LI>this.getState() == IN_SERVICE<br>
       </OL><B>Post-conditions:</B><OL>
        <LI>this.getState() == IN_SERVICE<br>
        <LI>Assume JcpCall call == createCall(); <br>
            call.getState() == IDLE <br>
            call.getConnections() == <CODE>null</CODE> <br>
       </OL>
        
       @return JcpCall object representing the new call.
       @throws InvalidStateException If the JcpProvider is not in the JcpProvider.IN_SERVICE state.
       @throws ResourceUnavailableException An internal resource necessary to create a new Call
       object is unavailable. 
       @throws PrivilegeViolationException If the application does not have the proper
       authority to create a new telephone call object. 
       @throws MethodNotSupportedException The implementation does not support creating
       new JcpCall objects. 
     */
    public JcpCall createCall()       throws InvalidStateException, ResourceUnavailableException, 
    PrivilegeViolationException, MethodNotSupportedException;

    /**
       Adds a listener to this provider. JcpProvider related events are reported via the
       JcpProviderListener interface. The JcpProvider object will report events to this 
       interface for the lifetime of the JcpProvider object or until the listener is 
       removed with the {@link #removeProviderListener(JcpProviderListener)} method or until the 
       JcpProvider is no longer observable. <p>
       
       If the JcpProvider becomes unobservable, a JcpProviderEvent with id 
       {@link JcpProviderEvent#PROVIDER_EVENT_TRANSMISSION_ENDED} is delivered to the application as a final
       event. No further events are delivered to the listener unless it is explicitly
        re-added by the application.<p>

       This method is valid anytime and has no pre-conditions. Application must have
       the ability to add listeners to JcpProviders so they can monitor the changes in 
       state in the JcpProvider. 
       When this method is invoked on a JcpProvider in the SHUTDOWN state, 
       the {@link JcpProviderEvent#PROVIDER_EVENT_TRANSMISSION_ENDED} 
       is delivered to the client right away. Further, the 
       {@link JcpProviderEvent#PROVIDER_EVENT_TRANSMISSION_ENDED} 
       event can be delivered even before this method returns. 
       
       <p>
       
       If an application attempts to add an instance of an listener already
       present on this JcpProvider, then repeated attempts to add the instance
       of the listener will silently fail, i.e. multiple instances of an 
       listener are not added and no exception will be thrown.

       <P><B>Post-Conditions:</B>
       <OL>
        <LI>A snapshot of events is delivered to the listener, if appropriate. 
       </OL>

       @param providerlistener JcpProviderListener object that receives the specified events.
       @throws MethodNotSupportedException This methods is not supported.
       @throws ResourceUnavailableException The resource limit for the number of 
       listeners has been exceeded. 
       <br>
       @see JcpProviderListener
     */
    public void addProviderListener( JcpProviderListener providerlistener )     throws ResourceUnavailableException, 
    MethodNotSupportedException;

    /**
       Removes the given listener from the provider. The given listener will no longer
       receive events generated by this JcpProvider object. The final event will have
       id {@link JcpProviderEvent#PROVIDER_EVENT_TRANSMISSION_ENDED}. 
       Also, if the listener is not currently registered with the JcpProvider, then this 
       method fails silently, i.e. no listener is removed and no exception is thrown. 

       <P><B>Post-Conditions:</B> <OL>
        <LI> JcpProviderEvent with id PROVIDER_EVENT_TRANSMISSION_ENDED 
        is delivered to listener.
        </OL>

       @param providerlistener JcpProviderListener object being removed.
     */
    public void removeProviderListener( JcpProviderListener providerlistener );


    /**
    Returns the unique string name of this JcpProvider instance. Each different JcpProvider must
    have a unique string associated with it. This is the same string which the 
    application passed to the {@link JcpPeer#getProvider(String)} method to create this JcpProvider
    instance. 
    @return The unique String name of this Provider. 
    */
    
    public String getName();
    /**
    Returns an {@link JcpAddress} object which corresponds to the (telephone) number string
    provided. If the provided name does not correspond to a JcpAddress known by the 
    JcpProvider and within the JcpProvider's domain, {@link InvalidArgumentException} is thrown.

       <P><B>Post-Conditions:</B> <OL>
        <LI>Let addres = this.getAddress(addr);
        <LI> Then (addres.getName()).equals(addr) returns true;
        </OL>

    @param address the address string which possibly represents a telephone number.
    @return The JcpAddress object which corresponds to the given number. 
    @throws InvalidPartyException This exception, with with type == {@link InvalidPartyException#UNKNOWN_PARTY}, 
    is thrown if the given number does not correspond to a valid JcpAddress under this JcpProvider's domain. 
    */
    public JcpAddress getAddress(String address) throws InvalidPartyException;
    
    
    /**
    Instructs the JcpProvider to shut itself down and provide all necessary cleanup.
    Applications invoke this method when they no
     longer intend to use the JcpProvider, most often right before they exit. 
     This method is intended to allow the JcpProvider to
     perform any necessary cleanup which would not be taken care of when the 
     Java objects are garbage collected. This
     method causes the JcpProvider to move into the {@link #SHUTDOWN} state, 
     in which it will stay indefinitely. <p>

     If the JcpProvider is already in the {@link #SHUTDOWN} state, this method does
     nothing. The invocation of this method
     should not affect other applications which are using the same 
     implementation of the JcpProvider object. <p>
     
     <B>Post-Conditions:</B> <OL>
     <LI> this.getState() == SHUTDOWN
     </OL>
    */
    public void shutdown();
    /**
        This state indicates that the JcpProvider is currently available for use. 
    */
    public static final int IN_SERVICE     = 1;

    /**
        This state indicates that the JcpProvider is currently not available for use. Providers
        may come back in service at any time. However, the application can take no direct
        action to cause this change.
    */
    public static final int OUT_OF_SERVICE = 2;

    /**
        This state indicates that the JcpProvider is permanently no longer available for use.
     */
    public static final int SHUTDOWN = 3;

}

